@@ -10,6 +10,7 @@ from . import xproto
 from . import render
 from . import shape
 class QueryVersionReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -37,6 +38,7 @@ class SelectionEventMask:
     SelectionWindowDestroy = 1 << 1
     SelectionClientClose = 1 << 2
 class SelectionNotifyEvent(xcffib.Event):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -68,6 +70,7 @@ class CursorNotify:
 class CursorNotifyMask:
     DisplayCursor = 1 << 0
 class CursorNotifyEvent(xcffib.Event):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -94,6 +97,7 @@ class CursorNotifyEvent(xcffib.Event):
         return self
 _events[1] = CursorNotifyEvent
 class GetCursorImageReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -105,6 +109,7 @@ class GetCursorImageReply(xcffib.Reply):
 class GetCursorImageCookie(xcffib.Cookie):
     reply_type = GetCursorImageReply
 class BadRegionError(xcffib.Error):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -122,6 +127,7 @@ _errors[0] = BadRegionError
 class Region:
     _None = 0
 class FetchRegionReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -136,6 +142,7 @@ class FetchRegionReply(xcffib.Reply):
 class FetchRegionCookie(xcffib.Cookie):
     reply_type = FetchRegionReply
 class GetCursorNameReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -147,6 +154,7 @@ class GetCursorNameReply(xcffib.Reply):
 class GetCursorNameCookie(xcffib.Cookie):
     reply_type = GetCursorNameReply
 class GetCursorImageAndNameReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -168,6 +176,7 @@ class ClientDisconnectFlags:
     Default = 0
     Terminate = 1 << 0
 class GetClientDisconnectModeReply(xcffib.Reply):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -244,10 +253,11 @@ class xfixesExtension(xcffib.Extension):
         buf = io.BytesIO()
         buf.write(struct.pack("=xx2xIII", source1, source2, destination))
         return self.send_request(15, buf, is_checked=is_checked)
-    def InvertRegion(self, source, destination, bounds, is_checked=False):
+    def InvertRegion(self, source, bounds, destination, is_checked=False):
         buf = io.BytesIO()
-        buf.write(struct.pack("=xx2xII", source, destination))
-        buf.write(bounds.pack() if hasattr(bounds, "pack") else RECTANGLE.synthetic(*bounds).pack())
+        buf.write(struct.pack("=xx2xI", source))
+        buf.write(bounds.pack() if hasattr(bounds, "pack") else xproto.RECTANGLE.synthetic(*bounds).pack())
+        buf.write(struct.pack("=I", destination))
         return self.send_request(16, buf, is_checked=is_checked)
     def TranslateRegion(self, region, dx, dy, is_checked=False):
         buf = io.BytesIO()
